@@ -25,11 +25,12 @@
  */
 
 #include <sys/cdefs.h>
-#include <sys/mman.h>
 #include <sys/queue.h>
 #include <sys/stat.h>
+
 #include <archive.h>
 #include <archive_entry.h>
+#include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <gelf.h>
@@ -40,10 +41,10 @@
 
 #include "ar.h"
 
-ELFTC_VCSID("$Id: write.c 2130 2011-11-10 06:56:46Z jkoshy $");
+ELFTC_VCSID("$Id: write.c 2496 2012-04-24 02:33:40Z jkoshy $");
 
-#define _ARMAG_LEN 8		/* length of ar magic string */
-#define _ARHDR_LEN 60		/* length of ar header */
+#define _ARMAG_LEN 8		/* length of the magic string */
+#define _ARHDR_LEN 60		/* length of the archive header */
 #define _INIT_AS_CAP 128	/* initial archive string table size */
 #define _INIT_SYMOFF_CAP (256*(sizeof(uint32_t))) /* initial so table size */
 #define _INIT_SYMNAME_CAP 1024			  /* initial sn table size */
@@ -55,65 +56,22 @@ static void	add_to_ar_str_table(struct bsdar *bsdar, const char *name);
 static void	add_to_ar_sym_table(struct bsdar *bsdar, const char *name);
 static struct ar_obj	*create_obj_from_file(struct bsdar *bsdar,
 		    const char *name, time_t mtime);
-static void	create_symtab_entry(struct bsdar *bsdar, void *maddr,
-		    size_t size);
-static void	free_obj(struct bsdar *bsdar, struct ar_obj *obj);
+static void	create_symtab_entry(struct bsdar *bsdar, Elf *e);
+static void	free_obj(struct ar_obj *obj);
 static void	insert_obj(struct bsdar *bsdar, struct ar_obj *obj,
 		    struct ar_obj *pos);
 static void	read_objs(struct bsdar *bsdar, const char *archive,
 		    int checkargv);
-static void	write_archive(struct bsdar *bsdar, char mode);
 static void	write_cleanup(struct bsdar *bsdar);
 static void	write_data(struct bsdar *bsdar, struct archive *a,
 		    const void *buf, size_t s);
 static void	write_objs(struct bsdar *bsdar);
 
-void
-ar_mode_d(struct bsdar *bsdar)
-{
-
-	write_archive(bsdar, 'd');
-}
-
-void
-ar_mode_m(struct bsdar *bsdar)
-{
-
-	write_archive(bsdar, 'm');
-}
-
-void
-ar_mode_q(struct bsdar *bsdar)
-{
-
-	write_archive(bsdar, 'q');
-}
-
-void
-ar_mode_r(struct bsdar *bsdar)
-{
-
-	write_archive(bsdar, 'r');
-}
-
-void
-ar_mode_s(struct bsdar *bsdar)
-{
-
-	write_archive(bsdar, 's');
-}
-
-void
-ar_mode_A(struct bsdar *bsdar)
-{
-
-	write_archive(bsdar, 'A');
-}
-
 /*
- * Create object from file, return created obj upon success, or NULL
- * when an error occurs or the member is not newer than existing
- * one while -u is specifed.
+ * Create an object from a file, and return the created object
+ * descriptor.  Return NULL if either an error occurs, or if the '-u'
+ * option was specifed and the member is not newer than the existing
+ * one in the archive.
  */
 static struct ar_obj *
 create_obj_from_file(struct bsdar *bsdar, const char *name, time_t mtime)
@@ -122,6 +80,7 @@ create_obj_from_file(struct bsdar *bsdar, const char *name, time_t mtime)
 	struct stat		 sb;
 	const char		*bname;
 	char			*tmpname;
+	int			fd;
 
 	if (name == NULL)
 		return (NULL);
@@ -129,7 +88,10 @@ create_obj_from_file(struct bsdar *bsdar, const char *name, time_t mtime)
 	obj = malloc(sizeof(struct ar_obj));
 	if (obj == NULL)
 		bsdar_errc(bsdar, errno, "malloc failed");
-	if ((obj->fd = open(name, O_RDONLY, 0)) < 0) {
+
+	obj->elf = NULL;
+
+	if ((fd = open(name, O_RDONLY, 0)) < 0) {
 		bsdar_warnc(bsdar, errno, "can't open file: %s", name);
 		free(obj);
 		return (NULL);
@@ -148,7 +110,7 @@ create_obj_from_file(struct bsdar *bsdar, const char *name, time_t mtime)
 		    bsdar_errc(bsdar, errno, "strdup failed");
 	free(tmpname);
 
-	if (fstat(obj->fd, &sb) < 0) {
+	if (fstat(fd, &sb) < 0) {
 		bsdar_warnc(bsdar, errno, "can't fstat file: %s", obj->name);
 		goto giveup;
 	}
@@ -164,19 +126,19 @@ create_obj_from_file(struct bsdar *bsdar, const char *name, time_t mtime)
 	}
 
 	/*
-	 * When option '-u' is specified and member is not newer than the
-	 * existing one, the replace will not happen. While if mtime == 0,
-	 * which indicates that this is to "replace a none exist member",
-	 * the replace will proceed regardless of '-u'.
+	 * If the '-u' option is specified and member is not newer
+	 * than the existing one, we should not replace the member.
+	 * However, if mtime == 0, i.e., if nonexistent members are to
+	 * be forcibly replaced, then the '-u' option is to be ignored.
 	 */
 	if (mtime != 0 && bsdar->options & AR_U && sb.st_mtime <= mtime)
 		goto giveup;
 
 	/*
-	 * When option '-D' is specified, mtime and UID / GID from the file
-	 * will be replaced with 0, and file mode with 644. This ensures that 
-	 * checksums will match for two archives containing the exact same
-	 * files.
+	 * When the '-D' option is specified, the mtime and UID/GID of
+	 * the member will be set to 0, and the file mode will be set
+	 * to 644. This ensures that checksums will match for two
+	 * archives containing identical content.
 	 */
 	if (bsdar->options & AR_D) {
 		obj->uid = 0;
@@ -194,23 +156,36 @@ create_obj_from_file(struct bsdar *bsdar, const char *name, time_t mtime)
 	obj->ino = sb.st_ino;
 
 	if (obj->size == 0) {
-		obj->maddr = NULL;
 		return (obj);
 	}
 
-	if ((obj->maddr = mmap(NULL, obj->size, PROT_READ,
-	    MAP_PRIVATE, obj->fd, (off_t)0)) == MAP_FAILED) {
-		bsdar_warnc(bsdar, errno, "can't mmap file: %s", obj->name);
+	if ((obj->elf = elf_open(fd)) == NULL) {
+		bsdar_warnc(bsdar, 0, "file initialization failed for %s: %s",
+		    obj->name, elf_errmsg(-1));
 		goto giveup;
 	}
-	if (close(obj->fd) < 0)
+
+	/*
+	 * Read the object fully into memory and close its file
+	 * descriptor.
+	 */
+	if (elf_cntl(obj->elf, ELF_C_FDREAD) < 0) {
+		bsdar_warnc(bsdar, 0, "%s could not be read in: %s",
+		    obj->name, elf_errmsg(-1));
+		goto giveup;
+	}
+
+	if (close(fd) < 0)
 		bsdar_errc(bsdar, errno, "close failed: %s",
 		    obj->name);
 
 	return (obj);
 
 giveup:
-	if (close(obj->fd) < 0)
+	if (obj->elf)
+		elf_end(obj->elf);
+
+	if (close(fd) < 0)
 		bsdar_errc(bsdar, errno, "close failed: %s",
 		    obj->name);
 	free(obj->name);
@@ -219,23 +194,21 @@ giveup:
 }
 
 /*
- * Free object itself and its associated allocations.
+ * Free an object and its associated allocations.
  */
 static void
-free_obj(struct bsdar *bsdar, struct ar_obj *obj)
+free_obj(struct ar_obj *obj)
 {
-	if (obj->fd == -1)
-		free(obj->maddr);
-	else
-		if (obj->maddr != NULL && munmap(obj->maddr, obj->size))
-			bsdar_warnc(bsdar, errno,
-			    "can't munmap file: %s", obj->name);
+	if (obj->elf)
+		elf_end(obj->elf);
+
 	free(obj->name);
 	free(obj);
 }
 
 /*
- * Insert obj to the tail, or before/after the pos obj.
+ * Insert an object into a list, either before/after the 'pos' obj or
+ * at the end of the list.
  */
 static void
 insert_obj(struct bsdar *bsdar, struct ar_obj *obj, struct ar_obj *pos)
@@ -245,8 +218,9 @@ insert_obj(struct bsdar *bsdar, struct ar_obj *obj, struct ar_obj *pos)
 
 	if (pos == NULL || obj == pos)
 		/*
-		 * If the object to move happens to be the posistion obj,
-		 * or if there is not a pos obj, move it to tail.
+		 * If the object to move happens to be the position
+		 * obj, or if there is no position obj, move the
+		 * object to the end.
 		 */
 		goto tail;
 
@@ -265,10 +239,10 @@ tail:
 }
 
 /*
- * Read objects from archive into v_obj list. Note that checkargv is
- * set when read_objs is used to read objects from the target of
- * ADDLIB command (ar script mode), in this case argv array possibly
- * specifies the members ADDLIB want.
+ * Read objects from archive into the 'v_obj' list. Note that
+ * 'checkargv' is set when read_objs() is used to read objects from
+ * the target of 'ADDLIB' command in ar script mode; in this case the
+ * 'argv' array specifies the members that 'ADDLIB' is to operate on.
  */
 static void
 read_objs(struct bsdar *bsdar, const char *archive, int checkargv)
@@ -304,14 +278,14 @@ read_objs(struct bsdar *bsdar, const char *archive, int checkargv)
 		name = archive_entry_pathname(entry);
 
 		/*
-		 * skip pseudo members.
+		 * Skip pseudo members.
 		 */
 		if (bsdar_is_pseudomember(bsdar, name))
 			continue;
 
 		/*
-		 * If checkargv is set, only read those members specified
-		 * in argv.
+		 * If 'checkargv' is set, only read those members
+		 * specified in argv.
 		 */
 		if (checkargv && bsdar->argc > 0) {
 			find = 0;
@@ -350,7 +324,18 @@ read_objs(struct bsdar *bsdar, const char *archive, int checkargv)
 		obj = malloc(sizeof(struct ar_obj));
 		if (obj == NULL)
 			bsdar_errc(bsdar, errno, "malloc failed");
-		obj->maddr = buff;
+		obj->elf = NULL;
+		if (buff) {
+			obj->elf = elf_openmemory(buff, size);
+			if (obj->elf == NULL) {
+				bsdar_warnc(bsdar, 0, "elf_openmemory() "
+				    "failed for %s: %s", name,
+				    elf_errmsg(-1));
+				free(buff);
+				free(obj);
+				continue;
+			}
+		}
 		if ((obj->name = strdup(name)) == NULL)
 			bsdar_errc(bsdar, errno, "strdup failed");
 		obj->size = size;
@@ -361,11 +346,6 @@ read_objs(struct bsdar *bsdar, const char *archive, int checkargv)
 		obj->dev = 0;
 		obj->ino = 0;
 
-		/*
-		 * Objects from archive have obj->fd set to -1,
-		 * for the ease of cleaning up.
-		 */
-		obj->fd = -1;
 		TAILQ_INSERT_TAIL(&bsdar->v_obj, obj, objs);
 	}
 	AC(archive_read_close(a));
@@ -373,10 +353,10 @@ read_objs(struct bsdar *bsdar, const char *archive, int checkargv)
 }
 
 /*
- * Determine the constitution of resulting archive.
+ * Write an archive.
  */
-static void
-write_archive(struct bsdar *bsdar, char mode)
+void
+ar_write_archive(struct bsdar *bsdar, int mode)
 {
 	struct ar_obj		 *nobj, *obj, *obj_temp, *pos;
 	struct stat		  sb;
@@ -389,9 +369,12 @@ write_archive(struct bsdar *bsdar, char mode)
 	pos = NULL;
 	memset(&sb, 0, sizeof(sb));
 
+	assert(mode == 'A' || mode == 'd' || mode == 'm' || mode == 'q' ||
+	    mode == 'r' || mode == 's');
+
 	/*
-	 * Test if the specified archive exists, to figure out
-	 * whether we are creating one here.
+	 * Test if the specified archive exists, to determine
+	 * whether we are creating a new archive.
 	 */
 	if (stat(bsdar->filename, &sb) != 0) {
 		if (errno != ENOENT) {
@@ -407,7 +390,7 @@ write_archive(struct bsdar *bsdar, char mode)
 			return;
 		}
 
-		/* Issue a warning if -c is not specified when creating. */
+		/* Issue a message if the '-c' option was not specified. */
 		if (!(bsdar->options & AR_C))
 			bsdar_warnc(bsdar, 0, "creating %s", bsdar->filename);
 		goto new_archive;
@@ -417,7 +400,7 @@ write_archive(struct bsdar *bsdar, char mode)
 	bsdar->ar_ino = sb.st_ino;
 
 	/*
-	 * First read members from existing archive.
+	 * First read members from the existing archive.
 	 */
 	read_objs(bsdar, bsdar->filename, 0);
 
@@ -436,16 +419,18 @@ write_archive(struct bsdar *bsdar, char mode)
 		goto new_archive;
 
 	/*
-	 * Mode 'A' adds the contents of another archive to the tail of
-	 * current archive. Note that mode 'A' is a special mode for the
-	 * ADDLIB command of the ar script mode. Currently there is no
-	 * access to this function from the ar command line mode.
+	 * Mode 'A' adds the contents of another archive to the tail
+	 * of current archive. Note that mode 'A' is a special mode
+	 * for the 'ADDLIB' command in ar's script mode. Currently
+	 * there is no option that invokes this function from ar's
+	 * command line.
 	 */
 	if (mode == 'A') {
 		/*
-		 * Read objects from the target archive of ADDLIB command.
-		 * If there are members spcified in argv, read those members
-		 * only, otherwise the entire archive will be read.
+		 * Read objects from the target archive of the
+		 * 'ADDLIB' command.  If there are members spcified in
+		 * 'argv', read those members only, otherwise the
+		 * entire archive will be read.
 		 */
 		read_objs(bsdar, bsdar->addlib, 1);
 		goto write_objs;
@@ -463,8 +448,9 @@ write_archive(struct bsdar *bsdar, char mode)
 		}
 
 		/*
-		 * If can't find `pos' specified by user,
-		 * sliently insert objects at tail.
+		 * If we cannot find the position specified by the
+		 * user, sliently insert objects at the tail of the
+		 * list.
 		 */
 		if (pos == NULL)
 			bsdar->options &= ~(AR_A | AR_B);
@@ -485,8 +471,8 @@ write_archive(struct bsdar *bsdar, char mode)
 
 			if (mode == 'r') {
 				/*
-				 * if the new member is not qualified
-				 * to replace the old one, skip it.
+				 * If the new member should not
+				 * replace the old one, skip it.
 				 */
 				nobj = create_obj_from_file(bsdar, *av,
 				    obj->mtime);
@@ -500,7 +486,7 @@ write_archive(struct bsdar *bsdar, char mode)
 
 			TAILQ_REMOVE(&bsdar->v_obj, obj, objs);
 			if (mode == 'd' || mode == 'r')
-				free_obj(bsdar, obj);
+				free_obj(obj);
 
 			if (mode == 'm')
 				insert_obj(bsdar, obj, pos);
@@ -516,10 +502,11 @@ write_archive(struct bsdar *bsdar, char mode)
 
 new_archive:
 	/*
-	 * When operating in mode 'r', directly add those user specified
-	 * objects which do not exist in current archive. When operating
-	 * in mode 'q', all objects specified in command line args are
-	 * appended to the archive, without comparing with existing ones.
+	 * When operating in mode 'r', directly add the specified
+	 * objects which do not exist in current archive. When
+	 * operating in mode 'q', all objects specified by the command
+	 * line args are appended to the archive, without checking
+	 * existing members in the archive.
 	 */
 	for (i = 0; i < bsdar->argc; i++) {
 		av = &bsdar->argv[i];
@@ -539,7 +526,7 @@ write_objs:
 }
 
 /*
- * Memory cleaning up.
+ * Release memory.
  */
 static void
 write_cleanup(struct bsdar *bsdar)
@@ -548,7 +535,7 @@ write_cleanup(struct bsdar *bsdar)
 
 	TAILQ_FOREACH_SAFE(obj, &bsdar->v_obj, objs, obj_temp) {
 		TAILQ_REMOVE(&bsdar->v_obj, obj, objs);
-		free_obj(bsdar, obj);
+		free_obj(obj);
 	}
 
 	free(bsdar->as);
@@ -674,18 +661,18 @@ write_objs(struct bsdar *bsdar)
 	size_t namelen;		/* size of member name. */
 	size_t obj_sz;		/* size of object + extended header. */
 	int			 i;
+	char			*buf;
 	const char		*entry_name;
 
-	if (elf_version(EV_CURRENT) == EV_NONE)
-		bsdar_errc(bsdar, 0, "ELF library initialization failed: %s",
-		    elf_errmsg(-1));
-
 	bsdar->rela_off = 0;
 
-	/* Create archive symbol table and archive string table, if need. */
+	/*
+	 * Create the archive symbol table and the archive string
+	 * table, if needed.
+	 */
 	TAILQ_FOREACH(obj, &bsdar->v_obj, objs) {
-		if (!(bsdar->options & AR_SS) && obj->maddr != NULL)
-			create_symtab_entry(bsdar, obj->maddr, obj->size);
+		if (!(bsdar->options & AR_SS) && obj->elf != NULL)
+			create_symtab_entry(bsdar, obj->elf);
 
 		obj_sz = 0;
 		namelen = strlen(obj->name);
@@ -704,24 +691,27 @@ write_objs(struct bsdar *bsdar)
 	}
 
 	/*
-	 * Pad the symbol name string table. It is treated specially because
-	 * symbol name table should be padded by a '\0', not the common '\n'
-	 * for other members. The size of sn table includes the pad bit.
+	 * Pad the symbol name string table. It is treated specially
+	 * because symbol name table should be padded by a '\0', and
+	 * not '\n' as for normal members. The size of the 'sn' table
+	 * includes the pad byte.
 	 */
 	if (bsdar->s_cnt != 0 && bsdar->s_sn_sz % 2 != 0)
 		bsdar->s_sn[bsdar->s_sn_sz++] = '\0';
 
 	/*
-	 * Archive string table is padded by a "\n" as the normal members.
-	 * The difference is that the size of archive string table counts
-	 * in the pad bit, while normal members' size fileds do not.
+	 * The archive string table is padded by a "\n" like a normal
+	 * member.  The difference is that the size of archive string
+	 * table includes the pad byte, while normal members' size
+	 * fields do not.
 	 */
 	if (bsdar->as != NULL && bsdar->as_sz % 2 != 0)
 		bsdar->as[bsdar->as_sz++] = '\n';
 
 	/*
-	 * If there is a symbol table, calculate the size of pseudo members,
-	 * convert previously stored relative offsets to absolute ones.
+	 * If there is a symbol table, calculate the size of pseudo
+	 * members, and convert previously stored relative offsets to
+	 * absolute ones.
 	 *
 	 * absolute_offset = relative_offset + size_of_pseudo_members)
 	 */
@@ -747,9 +737,9 @@ write_objs(struct bsdar *bsdar)
 	AC(archive_write_open_filename(a, bsdar->filename));
 
 	/*
-	 * write the archive symbol table, if there is one.
-	 * If options -s is explicitly specified or we are invoked
-	 * as ranlib, write the symbol table even if it is empty.
+	 * Write the archive symbol table, if there is one.  If
+	 * options '-s' was explicitly specified or if we were invoked
+	 * as 'ranlib', write the symbol table even if it is empty.
 	 */
 	if ((bsdar->s_cnt != 0 && !(bsdar->options & AR_SS)) ||
 	    bsdar->options & AR_S) {
@@ -771,7 +761,7 @@ write_objs(struct bsdar *bsdar)
 		archive_entry_free(entry);
 	}
 
-	/* write the archive string table, if any. */
+	/* Write the archive string table, if any. */
 	if (bsdar->as != NULL) {
 		entry = archive_entry_new();
 		archive_entry_copy_pathname(entry, AR_STRINGTAB_NAME_SVR4);
@@ -781,8 +771,14 @@ write_objs(struct bsdar *bsdar)
 		archive_entry_free(entry);
 	}
 
-	/* write normal members. */
+	/* Write normal members. */
 	TAILQ_FOREACH(obj, &bsdar->v_obj, objs) {
+		if ((buf = elf_rawfile(obj->elf, NULL)) == NULL) {
+			bsdar_warnc(bsdar, 0, "elf_rawfile() failed: %s",
+			    elf_errmsg(-1));
+			continue;
+		}
+
 		entry = archive_entry_new();
 		archive_entry_copy_pathname(entry, obj->name);
 		archive_entry_set_uid(entry, obj->uid);
@@ -794,7 +790,7 @@ write_objs(struct bsdar *bsdar)
 		archive_entry_set_ino(entry, obj->ino);
 		archive_entry_set_filetype(entry, AE_IFREG);
 		AC(archive_write_header(a, entry));
-		write_data(bsdar, a, obj->maddr, obj->size);
+		write_data(bsdar, a, buf, obj->size);
 		archive_entry_free(entry);
 	}
 
@@ -806,9 +802,8 @@ write_objs(struct bsdar *bsdar)
  * Extract global symbols from ELF binary members.
  */
 static void
-create_symtab_entry(struct bsdar *bsdar, void *maddr, size_t size)
+create_symtab_entry(struct bsdar *bsdar, Elf *e)
 {
-	Elf		*e;
 	Elf_Scn		*scn;
 	GElf_Shdr	 shdr;
 	GElf_Sym	 sym;
@@ -817,20 +812,13 @@ create_symtab_entry(struct bsdar *bsdar, void *maddr, size_t size)
 	size_t		 n, shstrndx;
 	int		 elferr, tabndx, len, i;
 
-	if ((e = elf_memory(maddr, size)) == NULL) {
-		bsdar_warnc(bsdar, 0, "elf_memory() failed: %s",
-		     elf_errmsg(-1));
-		return;
-	}
 	if (elf_kind(e) != ELF_K_ELF) {
-		/* Sliently ignore non-elf member. */
-		elf_end(e);
+		/* Silently a ignore non-ELF member. */
 		return;
 	}
 	if (elf_getshstrndx(e, &shstrndx) == 0) {
 		bsdar_warnc(bsdar, 0, "elf_getshstrndx failed: %s",
 		     elf_errmsg(-1));
-		elf_end(e);
 		return;
 	}
 
@@ -858,7 +846,6 @@ create_symtab_entry(struct bsdar *bsdar, void *maddr, size_t size)
 		     elf_errmsg(elferr));
 	if (tabndx == -1) {
 		bsdar_warnc(bsdar, 0, "can't find .strtab section");
-		elf_end(e);
 		return;
 	}
 
@@ -885,12 +872,12 @@ create_symtab_entry(struct bsdar *bsdar, void *maddr, size_t size)
 					continue;
 				}
 
-				/* keep only global or weak symbols */
+				/* Keep only global and weak symbols. */
 				if (GELF_ST_BIND(sym.st_info) != STB_GLOBAL &&
 				    GELF_ST_BIND(sym.st_info) != STB_WEAK)
 					continue;
 
-				/* keep only defined symbols */
+				/* Keep only defined symbols. */
 				if (sym.st_shndx == SHN_UNDEF)
 					continue;
 
@@ -910,8 +897,6 @@ create_symtab_entry(struct bsdar *bsdar, void *maddr, size_t size)
 	if (elferr != 0)
 		bsdar_warnc(bsdar, 0, "elf_nextscn failed: %s",
 		     elf_errmsg(elferr));
-
-	elf_end(e);
 }
 
 /*
@@ -929,8 +914,9 @@ add_to_ar_str_table(struct bsdar *bsdar, const char *name)
 	}
 
 	/*
-	 * The space required for holding one member name in as table includes:
-	 * strlen(name) + (1 for '/') + (1 for '\n') + (possibly 1 for padding).
+	 * The space required for holding one member name in the 'as'
+	 * table includes: strlen(name) + (1 for '/') + (1 for '\n') +
+	 * (possibly 1 for padding).
 	 */
 	while (bsdar->as_sz + strlen(name) + 3 > bsdar->as_cap) {
 		bsdar->as_cap *= 2;
@@ -976,8 +962,9 @@ add_to_ar_sym_table(struct bsdar *bsdar, const char *name)
 	bsdar->s_cnt++;
 
 	/*
-	 * The space required for holding one symbol name in sn table includes:
-	 * strlen(name) + (1 for '\n') + (possibly 1 for padding).
+	 * The space required for holding one symbol name in the 'sn'
+	 * table includes: strlen(name) + (1 for '\n') + (possibly 1
+	 * for padding).
 	 */
 	while (bsdar->s_sn_sz + strlen(name) + 2 > bsdar->s_sn_cap) {
 		bsdar->s_sn_cap *= 2;
