@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2010 Kai Wang
+ * Copyright (c) 2010,2011 Kai Wang
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,7 @@
 
 #include "elfcopy.h"
 
-ELFTC_VCSID("$Id: ascii.c 2126 2011-11-10 05:07:18Z jkoshy $");
+ELFTC_VCSID("$Id: ascii.c 2358 2011-12-19 18:22:32Z kaiwang27 $");
 
 static void append_data(struct section *s, const void *buf, size_t sz);
 static char hex_digit(uint8_t n);
@@ -200,7 +200,7 @@ create_elf_from_srec(struct elfcopy *ecp, int ifd)
 	char line[_LINE_BUFSZ], name[_LINE_BUFSZ];
 	uint8_t data[_DATA_BUFSZ];
 	GElf_Ehdr oeh;
-	struct section *s, *sec, *sec_temp, *shtab;
+	struct section *s, *shtab;
 	FILE *ifp;
 	uint64_t addr, entry, off, sec_addr;
 	uintmax_t st_value;
@@ -209,13 +209,6 @@ create_elf_from_srec(struct elfcopy *ecp, int ifd)
 	char *rlt;
 	char type;
 
-	/* Reset internal section list. */
-	if (!TAILQ_EMPTY(&ecp->v_sec))
-		TAILQ_FOREACH_SAFE(sec, &ecp->v_sec, sec_list, sec_temp) {
-			TAILQ_REMOVE(&ecp->v_sec, sec, sec_list);
-			free(sec);
-		}
-
 	if ((_ifd = dup(ifd)) < 0)
 		err(EXIT_FAILURE, "dup failed");
 	if ((ifp = fdopen(_ifd, "r")) == NULL)
@@ -410,6 +403,9 @@ done:
 	if (elf_update(ecp->eout, ELF_C_WRITE) < 0)
 		errx(EXIT_FAILURE, "elf_update() failed: %s",
 		    elf_errmsg(-1));
+
+	/* Release allocated resource. */
+	free_elf(ecp);
 }
 
 void
@@ -477,20 +473,13 @@ create_elf_from_ihex(struct elfcopy *ecp, int ifd)
 	char line[_LINE_BUFSZ];
 	uint8_t data[_DATA_BUFSZ];
 	GElf_Ehdr oeh;
-	struct section *s, *sec, *sec_temp, *shtab;
+	struct section *s, *shtab;
 	FILE *ifp;
 	uint64_t addr, addr_base, entry, num, off, rec_addr, sec_addr;
 	size_t sz;
 	int _ifd, first, sec_index;
 	char type;
 
-	/* Reset internal section list. */
-	if (!TAILQ_EMPTY(&ecp->v_sec))
-		TAILQ_FOREACH_SAFE(sec, &ecp->v_sec, sec_list, sec_temp) {
-			TAILQ_REMOVE(&ecp->v_sec, sec, sec_list);
-			free(sec);
-		}
-
 	if ((_ifd = dup(ifd)) < 0)
 		err(EXIT_FAILURE, "dup failed");
 	if ((ifp = fdopen(_ifd, "r")) == NULL)
@@ -641,6 +630,9 @@ done:
 	if (elf_update(ecp->eout, ELF_C_WRITE) < 0)
 		errx(EXIT_FAILURE, "elf_update() failed: %s",
 		    elf_errmsg(-1));
+
+	/* Release allocated resource. */
+	free_elf(ecp);
 }
 
 #define	_SEC_NAMESZ	64
@@ -656,8 +648,8 @@ new_data_section(struct elfcopy *ecp, int sec_index, uint64_t off,
 		errx(EXIT_FAILURE, "malloc failed");
 	snprintf(name, _SEC_NAMESZ, ".sec%d", sec_index);
 
-	return (create_external_section(ecp, name, NULL, 0, off, SHT_PROGBITS,
-	    ELF_T_BYTE, SHF_ALLOC | SHF_WRITE, 1, addr, 0));
+	return (create_external_section(ecp, name, name, NULL, 0, off,
+		SHT_PROGBITS, ELF_T_BYTE, SHF_ALLOC | SHF_WRITE, 1, addr, 0));
 }
 
 static void
