@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2007-2010 Kai Wang
+ * Copyright (c) 2007-2011 Kai Wang
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,7 +27,7 @@
 #include <sys/cdefs.h>
 #include <sys/param.h>
 #include <sys/stat.h>
-#include <sys/time.h>
+
 #include <err.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -40,7 +40,7 @@
 
 #include "elfcopy.h"
 
-ELFTC_VCSID("$Id: main.c 2126 2011-11-10 05:07:18Z jkoshy $");
+ELFTC_VCSID("$Id: main.c 2372 2011-12-30 07:06:00Z jkoshy $");
 
 enum options
 {
@@ -78,6 +78,13 @@ enum options
 	ECP_WEAKEN_SYMBOLS
 };
 
+static struct option mcs_longopts[] =
+{
+	{ "help", no_argument, NULL, 'h' },
+	{ "version", no_argument, NULL, 'V' },
+	{ NULL, 0, NULL, 0 }
+};
+
 static struct option strip_longopts[] =
 {
 	{"discard-all", no_argument, NULL, 'x'},
@@ -202,7 +209,8 @@ static struct {
 	{NULL, 0}
 };
 
-static int	copy_tempfile(int fd, const char *out);
+static int	copy_from_tempfile(const char *src, const char *dst,
+    int infd, int *outfd);
 static void	create_file(struct elfcopy *ecp, const char *src,
     const char *dst);
 static void	elfcopy_main(struct elfcopy *ecp, int argc, char **argv);
@@ -265,7 +273,6 @@ static void	strip_usage(void);
 void
 create_elf(struct elfcopy *ecp)
 {
-	struct section	*sec, *sec_temp;
 	struct section	*shtab;
 	GElf_Ehdr	 ieh;
 	GElf_Ehdr	 oeh;
@@ -273,13 +280,6 @@ create_elf(struct elfcopy *ecp)
 
 	ecp->flags |= SYMTAB_INTACT;
 
-	/* Reset internal section list. */
-	if (!TAILQ_EMPTY(&ecp->v_sec))
-		TAILQ_FOREACH_SAFE(sec, &ecp->v_sec, sec_list, sec_temp) {
-			TAILQ_REMOVE(&ecp->v_sec, sec, sec_list);
-			free(sec);
-		}
-
 	/* Create EHDR. */
 	if (gelf_getehdr(ecp->ein, &ieh) == NULL)
 		errx(EXIT_FAILURE, "gelf_getehdr() failed: %s",
@@ -440,6 +440,41 @@ create_elf(struct elfcopy *ecp)
 	if (elf_update(ecp->eout, ELF_C_WRITE) < 0)
 		errx(EXIT_FAILURE, "elf_update() failed: %s",
 		    elf_errmsg(-1));
+
+	/* Release allocated resource. */
+	free_elf(ecp);
+}
+
+void
+free_elf(struct elfcopy *ecp)
+{
+	struct segment	*seg, *seg_temp;
+	struct section	*sec, *sec_temp;
+
+	/* Free internal segment list. */
+	if (!STAILQ_EMPTY(&ecp->v_seg)) {
+		STAILQ_FOREACH_SAFE(seg, &ecp->v_seg, seg_list, seg_temp) {
+			STAILQ_REMOVE(&ecp->v_seg, seg, segment, seg_list);
+			free(seg);
+		}
+	}
+
+	/* Free symbol table buffers. */
+	free_symtab(ecp);
+
+	/* Free internal section list. */
+	if (!TAILQ_EMPTY(&ecp->v_sec)) {
+		TAILQ_FOREACH_SAFE(sec, &ecp->v_sec, sec_list, sec_temp) {
+			TAILQ_REMOVE(&ecp->v_sec, sec, sec_list);
+			if (sec->buf != NULL)
+				free(sec->buf);
+			if (sec->newname != NULL)
+				free(sec->newname);
+			if (sec->pad != NULL)
+				free(sec->pad);
+			free(sec);
+		}
+	}
 }
 
 /* Create a temporary file. */
@@ -485,47 +520,60 @@ create_tempfile(char **fn, int *fd)
 }
 
 static int
-copy_tempfile(int fd, const char *out)
+copy_from_tempfile(const char *src, const char *dst, int infd, int *outfd)
 {
-	struct stat sb;
-	char *buf;
-	int out_fd, bytes;
+	int tmpfd;
 
-	if (fstat(fd, &sb) == -1)
-		err(EXIT_FAILURE, "fstat tmpfile failed");
-	if ((buf = malloc((size_t) sb.st_blksize)) == NULL)
-		err(EXIT_FAILURE, "malloc failed");
-	if (unlink(out) < 0)
-		err(EXIT_FAILURE, "unlink %s failed", out);
-	if ((out_fd = open(out, O_CREAT | O_WRONLY, 0755)) == -1)
-		err(EXIT_FAILURE, "create %s failed", out);
+	/*
+	 * First, check if we can use rename().
+	 */
+	if (rename(src, dst) >= 0) {
+		*outfd = infd;
+		return (0);
+	} else if (errno != EXDEV)
+		return (-1);
 
-	if (lseek(fd, 0, SEEK_SET) < 0)
-		err(EXIT_FAILURE, "lseek tmpfile failed");
-	while ((bytes = read(fd, buf, sb.st_blksize)) > 0) {
-		if (write(out_fd, buf, (size_t) bytes) != bytes) {
-			(void) unlink(out);
-			err(EXIT_FAILURE, "write %s failed", out);
-		}
-	}
-	if (bytes < 0) {
-		(void) unlink(out);
-		err(EXIT_FAILURE, "read failed");
-	}
+	/*
+	 * If the rename() failed due to 'src' and 'dst' residing in
+	 * two different file systems, invoke a helper function in
+	 * libelftc to do the copy.
+	 */
 
-	free(buf);
-	close(fd);
+	if (unlink(dst) < 0)
+		return (-1);
 
-	return (out_fd);
+	if ((tmpfd = open(dst, O_CREAT | O_WRONLY, 0755)) < 0)
+		return (-1);
+
+	if (lseek(infd, 0, SEEK_SET) < 0)
+		return (-1);
+
+	if (elftc_copyfile(infd, tmpfd) < 0)
+		return (-1);
+
+	/*
+	 * Remove the temporary file from the file system
+	 * namespace, and close its file descriptor.
+	 */
+	if (unlink(src) < 0)
+		return (-1);
+
+	(void) close(infd);
+
+	/*
+	 * Return the file descriptor for the destination.
+	 */
+	*outfd = tmpfd;
+
+	return (0);
 }
 
 static void
 create_file(struct elfcopy *ecp, const char *src, const char *dst)
 {
 	struct stat	 sb;
-	struct timeval	 tv[2];
 	char		*tempfile, *elftemp;
-	int		 ifd, ofd, ofd0, efd;
+	int		 efd, ifd, ofd, ofd0, tfd;
 
 	tempfile = NULL;
 
@@ -619,6 +667,14 @@ create_file(struct elfcopy *ecp, const char *src, const char *dst)
 			}
 			create_tempfile(&tempfile, &ofd0);
 
+
+			/*
+			 * Rewind the file descriptor being processed.
+			 */
+			if (lseek(ofd, 0, SEEK_SET) < 0)
+				err(EXIT_FAILURE,
+				    "lseek failed for the output object");
+
 			/*
 			 * Call flavour-specific conversion routine.
 			 */
@@ -660,30 +716,22 @@ copy_done:
 	if (tempfile != NULL) {
 		if (dst == NULL)
 			dst = src;
-		if (rename(tempfile, dst) == -1) {
-			if (errno == EXDEV) {
-				ofd = copy_tempfile(ofd, dst);
-				if (unlink(tempfile) < 0)
-					err(EXIT_FAILURE, "unlink %s failed",
-					    tempfile);
-			} else
-				err(EXIT_FAILURE, "rename %s to %s failed",
-				    tempfile, dst);
-		}
+
+		if (copy_from_tempfile(tempfile, dst, ofd, &tfd) < 0)
+			err(EXIT_FAILURE, "creation of %s failed", dst);
+
 		free(tempfile);
+		tempfile = NULL;
+
+		ofd = tfd;
 	}
 
 	if (strcmp(dst, "/dev/null") && fchmod(ofd, sb.st_mode) == -1)
 		err(EXIT_FAILURE, "fchmod %s failed", dst);
 
-	if (ecp->flags & PRESERVE_DATE) {
-		tv[0].tv_sec = sb.st_atime;
-		tv[0].tv_usec = 0;
-		tv[1].tv_sec = sb.st_mtime;
-		tv[1].tv_usec = 0;
-		if (futimes(ofd, tv) == -1)
-			err(EXIT_FAILURE, "futimes failed");
-	}
+	if ((ecp->flags & PRESERVE_DATE) &&
+	    elftc_set_timestamps(dst, &sb) < 0)
+		err(EXIT_FAILURE, "setting timestamps failed");
 
 	close(ifd);
 	close(ofd);
@@ -697,7 +745,7 @@ elfcopy_main(struct elfcopy *ecp, int argc, char **argv)
 	char			*fn, *s;
 	int			 opt;
 
-	while ((opt = getopt_long(argc, argv, "dB:gG:I:j:K:L:N:O:pR:sSwW:xXV",
+	while ((opt = getopt_long(argc, argv, "dB:gG:I:j:K:L:N:O:pR:s:SwW:xXV",
 	    elfcopy_longopts, NULL)) != -1) {
 		switch(opt) {
 		case 'B':
@@ -911,7 +959,8 @@ mcs_main(struct elfcopy *ecp, int argc, char **argv)
 
 	append = delete = compress = name = print = 0;
 	string = NULL;
-	while ((opt = getopt(argc, argv, "a:cdn:pV")) != -1) {
+	while ((opt = getopt_long(argc, argv, "a:cdhn:pV", mcs_longopts,
+		NULL)) != -1) {
 		switch(opt) {
 		case 'a':
 			append = 1;
@@ -933,6 +982,7 @@ mcs_main(struct elfcopy *ecp, int argc, char **argv)
 		case 'V':
 			print_version();
 			break;
+		case 'h':
 		default:
 			mcs_usage();
 		}
@@ -993,7 +1043,7 @@ strip_main(struct elfcopy *ecp, int argc, char **argv)
 	int			 i;
 
 	outfile = NULL;
-	while ((opt = getopt_long(argc, argv, "I:K:N:o:O:pR:sSdgVxXw",
+	while ((opt = getopt_long(argc, argv, "hI:K:N:o:O:pR:sSdgVxXw",
 	    strip_longopts, NULL)) != -1) {
 		switch(opt) {
 		case 'R':
@@ -1043,6 +1093,7 @@ strip_main(struct elfcopy *ecp, int argc, char **argv)
 		case ECP_STRIP_UNNEEDED:
 			ecp->strip = STRIP_UNNEEDED;
 			break;
+		case 'h':
 		default:
 			strip_usage();
 		}
@@ -1220,7 +1271,7 @@ process_symfile:
 static void
 set_input_target(struct elfcopy *ecp, const char *target_name)
 {
-	Bfd_Target *tgt;
+	Elftc_Bfd_Target *tgt;
 
 	if ((tgt = elftc_bfd_find_target(target_name)) == NULL)
 		errx(EXIT_FAILURE, "%s: invalid target name", target_name);
@@ -1230,7 +1281,7 @@ set_input_target(struct elfcopy *ecp, const char *target_name)
 static void
 set_output_target(struct elfcopy *ecp, const char *target_name)
 {
-	Bfd_Target *tgt;
+	Elftc_Bfd_Target *tgt;
 
 	if ((tgt = elftc_bfd_find_target(target_name)) == NULL)
 		errx(EXIT_FAILURE, "%s: invalid target name", target_name);
@@ -1259,7 +1310,7 @@ set_osabi(struct elfcopy *ecp, const char *abi)
 		errx(EXIT_FAILURE, "unrecognized OSABI %s", abi);
 }
 
-static const char *elfcopy_usagemsg = "\
+#define	ELFCOPY_USAGE_MESSAGE	"\
 Usage: %s [options] infile [outfile]\n\
   Transform an ELF object.\n\n\
   Options:\n\
@@ -1328,36 +1379,35 @@ Usage: %s [options] infile [outfile]\n\
   --set-start=ADDRESS          Set the start address of the ELF object.\n\
   --srec-forceS3               Only generate S3 S-Records.\n\
   --srec-len=LEN               Set the maximum length of a S-Record line.\n\
-  --strip-unneeded             Do not copy relocation information.\n\
-";
+  --strip-unneeded             Do not copy relocation information.\n"
 
 static void
-elfcopy_usage()
+elfcopy_usage(void)
 {
-	(void) fprintf(stderr, elfcopy_usagemsg, ELFTC_GETPROGNAME());
+	(void) fprintf(stderr, ELFCOPY_USAGE_MESSAGE, ELFTC_GETPROGNAME());
 	exit(EXIT_FAILURE);
 }
 
-static const char *mcs_usagemsg = "\
+#define	MCS_USAGE_MESSAGE	"\
 Usage: %s [options] file...\n\
   Manipulate the comment section in an ELF object.\n\n\
   Options:\n\
-  -a STRING    Append 'STRING' to the comment section.\n\
-  -c           Remove duplicate entries from the comment section.\n\
-  -d           Delete the comment section.\n\
-  -n NAME      Operate on the ELF section with name 'NAME'.\n\
-  -p           Print the contents of the comment section.\n\
-  -V           Print a version identifier and exit.\n\
-";
+  -a STRING        Append 'STRING' to the comment section.\n\
+  -c               Remove duplicate entries from the comment section.\n\
+  -d               Delete the comment section.\n\
+  -h | --help      Print a help message and exit.\n\
+  -n NAME          Operate on the ELF section with name 'NAME'.\n\
+  -p               Print the contents of the comment section.\n\
+  -V | --version   Print a version identifier and exit.\n"
 
 static void
-mcs_usage()
+mcs_usage(void)
 {
-	(void) fprintf(stderr, mcs_usagemsg, ELFTC_GETPROGNAME());
+	(void) fprintf(stderr, MCS_USAGE_MESSAGE, ELFTC_GETPROGNAME());
 	exit(EXIT_FAILURE);
 }
 
-static const char *strip_usagemsg = "\
+#define	STRIP_USAGE_MESSAGE	"\
 Usage: %s [options] file...\n\
   Discard information from ELF objects.\n\n\
   Options:\n\
@@ -1376,13 +1426,12 @@ Usage: %s [options] file...\n\
   -O TGT | --output-target=TGT    Set the output file format to 'TGT'.\n\
   -R SEC | --remove-section=SEC   Remove the section named 'SEC'.\n\
   -V | --version                  Print a version identifier and exit.\n\
-  -X | --discard-locals           Remove compiler-generated local symbols.\n\
-";
+  -X | --discard-locals           Remove compiler-generated local symbols.\n"
 
 static void
-strip_usage()
+strip_usage(void)
 {
-	(void) fprintf(stderr, strip_usagemsg, ELFTC_GETPROGNAME());
+	(void) fprintf(stderr, STRIP_USAGE_MESSAGE, ELFTC_GETPROGNAME());
 	exit(EXIT_FAILURE);
 }
 
@@ -1402,9 +1451,9 @@ main(int argc, char **argv)
 		errx(EXIT_FAILURE, "ELF library initialization failed: %s",
 		    elf_errmsg(-1));
 
-	ecp = malloc(sizeof(*ecp));
+	ecp = calloc(1, sizeof(*ecp));
 	if (ecp == NULL)
-		err(EXIT_FAILURE, "malloc failed");
+		err(EXIT_FAILURE, "calloc failed");
 	memset(ecp, 0, sizeof(*ecp));
 
 	ecp->itf = ecp->otf = ETF_ELF;
@@ -1433,6 +1482,8 @@ main(int argc, char **argv)
 	else
 		elfcopy_main(ecp, argc, argv);
 
+	free_sec_add(ecp);
+	free_sec_act(ecp);
 	free(ecp);
 
 	exit(EXIT_SUCCESS);
