@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2007-2010 Kai Wang
+ * Copyright (c) 2007-2011 Kai Wang
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,7 +35,7 @@
 
 #include "elfcopy.h"
 
-ELFTC_VCSID("$Id: sections.c 2126 2011-11-10 05:07:18Z jkoshy $");
+ELFTC_VCSID("$Id: sections.c 2358 2011-12-19 18:22:32Z kaiwang27 $");
 
 static void	add_gnu_debuglink(struct elfcopy *ecp);
 static uint32_t calc_crc32(const char *p, size_t len, uint32_t crc);
@@ -97,8 +97,8 @@ is_remove_section(struct elfcopy *ecp, const char *name)
 }
 
 /*
- * Relocation section need to be remove if the section it applies will
- * be removed.
+ * Relocation section needs to be removed if the section it applies to
+ * will be removed.
  */
 int
 is_remove_reloc_sec(struct elfcopy *ecp, uint32_t sh_info)
@@ -166,7 +166,6 @@ static void
 check_section_rename(struct elfcopy *ecp, struct section *s)
 {
 	struct sec_action *sac;
-	char *newname;
 	char *prefix;
 	size_t namelen;
 
@@ -190,10 +189,10 @@ check_section_rename(struct elfcopy *ecp, struct section *s)
 
 	if (prefix != NULL) {
 		namelen = strlen(s->name) + strlen(prefix) + 1;
-		if ((newname = malloc(namelen)) == NULL)
+		if ((s->newname = malloc(namelen)) == NULL)
 			err(EXIT_FAILURE, "malloc failed");
-		snprintf(newname, namelen, "%s%s", prefix, s->name);
-		s->name = newname;
+		snprintf(s->newname, namelen, "%s%s", prefix, s->name);
+		s->name = s->newname;
 	}
 }
 
@@ -282,6 +281,17 @@ lookup_sec_act(struct elfcopy *ecp, const char *name, int add)
 	return (sac);
 }
 
+void
+free_sec_act(struct elfcopy *ecp)
+{
+	struct sec_action *sac, *sac_temp;
+
+	STAILQ_FOREACH_SAFE(sac, &ecp->v_sac, sac_list, sac_temp) {
+		STAILQ_REMOVE(&ecp->v_sac, sac, sec_action, sac_list);
+		free(sac);
+	}
+}
+
 void
 insert_to_sec_list(struct elfcopy *ecp, struct section *sec, int tail)
 {
@@ -1035,9 +1045,9 @@ copy_data(struct section *s)
 }
 
 struct section *
-create_external_section(struct elfcopy *ecp, const char *name, void *buf,
-    uint64_t size, uint64_t off, uint64_t stype, Elf_Type dtype, uint64_t flags,
-    uint64_t align, uint64_t vma, int loadable)
+create_external_section(struct elfcopy *ecp, const char *name, char *newname,
+    void *buf, uint64_t size, uint64_t off, uint64_t stype, Elf_Type dtype,
+    uint64_t flags, uint64_t align, uint64_t vma, int loadable)
 {
 	struct section	*s;
 	Elf_Scn		*os;
@@ -1050,6 +1060,7 @@ create_external_section(struct elfcopy *ecp, const char *name, void *buf,
 	if ((s = calloc(1, sizeof(*s))) == NULL)
 		err(EXIT_FAILURE, "calloc failed");
 	s->name = name;
+	s->newname = newname;	/* needs to be free()'ed */
 	s->off = off;
 	s->sz = size;
 	s->vma = vma;
@@ -1117,7 +1128,7 @@ insert_sections(struct elfcopy *ecp)
 
 		/* TODO: Add section header vma/lma, flag changes here */
 
-		(void) create_external_section(ecp, sa->name, sa->content,
+		(void) create_external_section(ecp, sa->name, NULL, sa->content,
 		    sa->size, off, SHT_PROGBITS, ELF_T_BYTE, 0, 1, 0, 0);
 	}
 }
@@ -1287,6 +1298,19 @@ add_section(struct elfcopy *ecp, const char *arg)
 	ecp->flags |= SEC_ADD;
 }
 
+void
+free_sec_add(struct elfcopy *ecp)
+{
+	struct sec_add *sa, *sa_temp;
+
+	STAILQ_FOREACH_SAFE(sa, &ecp->v_sadd, sadd_list, sa_temp) {
+		STAILQ_REMOVE(&ecp->v_sadd, sa, sec_add, sadd_list);
+		free(sa->name);
+		free(sa->content);
+		free(sa);
+	}
+}
+
 static void
 add_gnu_debuglink(struct elfcopy *ecp)
 {
