@@ -28,11 +28,15 @@
 
 #include <sys/cdefs.h>
 #include <sys/param.h>
+#include <sys/queue.h>
 #include <sys/stat.h>
+
 #include <ar.h>
+#include <assert.h>
 #include <err.h>
 #include <fcntl.h>
 #include <gelf.h>
+#include <getopt.h>
 #include <libelftc.h>
 #include <inttypes.h>
 #include <stdio.h>
@@ -47,7 +51,7 @@
 
 #include "_elftc.h"
 
-ELFTC_VCSID("$Id: elfdump.c 2127 2011-11-10 05:32:07Z jkoshy $");
+ELFTC_VCSID("$Id: elfdump.c 2355 2011-12-19 16:42:26Z jkoshy $");
 
 #if defined(ELFTC_NEED_ELF_NOTE_DEFINITION)
 #include "native-elf-format.h"
@@ -815,6 +819,7 @@ static const char *get_symbol_name(struct elfdump *ed, int symtab, int i);
 static const char *get_string(struct elfdump *ed, int strtab, size_t off);
 static void	get_versym(struct elfdump *ed, int i, uint16_t **vs, int *nvs);
 static void	load_sections(struct elfdump *ed);
+static void	unload_sections(struct elfdump *ed);
 static void	usage(void);
 #ifdef	USE_LIBARCHIVE_AR
 static int	ac_detect_ar(int fd);
@@ -823,6 +828,13 @@ static void	ac_print_ar(struct elfdump *ed, int fd);
 static void	elf_print_ar(struct elfdump *ed, int fd);
 #endif	/* USE_LIBARCHIVE_AR */
 
+static struct option elfdump_longopts[] =
+{
+	{ "help",	no_argument,	NULL,	'H' },
+	{ "version",	no_argument,	NULL,	'V' },
+	{ NULL,		0,		NULL,	0   }
+};
+
 int
 main(int ac, char **av)
 {
@@ -834,7 +846,8 @@ main(int ac, char **av)
 	memset(ed, 0, sizeof(*ed));
 	STAILQ_INIT(&ed->snl);
 	ed->out = stdout;
-	while ((ch = getopt(ac, av, "acdeiGhknN:prsSvVw:")) != -1)
+	while ((ch = getopt_long(ac, av, "acdeiGHhknN:prsSvVw:",
+		elfdump_longopts, NULL)) != -1)
 		switch (ch) {
 		case 'a':
 			ed->options = ED_ALL;
@@ -891,6 +904,7 @@ main(int ac, char **av)
 				err(EXIT_FAILURE, "%s", optarg);
 			break;
 		case '?':
+		case 'H':
 		default:
 			usage();
 		}
@@ -898,7 +912,7 @@ main(int ac, char **av)
 	ac -= optind;
 	av += optind;
 
-	if (ed->flags & SOLARIS_FMT && ed->options == 0)
+	if (ed->options == 0)
 		ed->options = ED_ALL;
 	sn = NULL;
 	if (ed->options & ED_SYMTAB &&
@@ -910,7 +924,7 @@ main(int ac, char **av)
 				ed->flags |= ONLY_ARSYM;
 		}
 	}
-	if (ac == 0 || ed->options == 0)
+	if (ac == 0)
 		usage();
 	if (ac > 1)
 		ed->flags |= PRINT_FILENAME;
@@ -1278,6 +1292,8 @@ elf_print_elf(struct elfdump *ed)
 		elf_print_hash(ed);
 	if (ed->options & ED_CHECKSUM)
 		elf_print_checksum(ed);
+
+	unload_sections(ed);
 }
 
 /*
@@ -1294,14 +1310,12 @@ load_sections(struct elfdump *ed)
 	size_t		 shstrndx, ndx;
 	int		 elferr;
 
+	assert(ed->sl == NULL);
+
 	if (!elf_getshnum(ed->elf, &ed->shnum)) {
 		warnx("elf_getshnum failed: %s", elf_errmsg(-1));
 		return;
 	}
-	if (ed->sl != NULL) {
-		free(ed->sl);
-		ed->sl = NULL;
-	}
 	if (ed->shnum == 0)
 		return;
 	if ((ed->sl = calloc(ed->shnum, sizeof(*ed->sl))) == NULL)
@@ -1353,6 +1367,18 @@ load_sections(struct elfdump *ed)
 		warnx("elf_nextscn failed: %s", elf_errmsg(elferr));
 }
 
+/*
+ * Release section related resources.
+ */
+static void
+unload_sections(struct elfdump *ed)
+{
+	if (ed->sl != NULL) {
+		free(ed->sl);
+		ed->sl = NULL;
+	}
+}
+
 /*
  * Add a name to the '-N' name list.
  */
@@ -1447,6 +1473,10 @@ get_string(struct elfdump *ed, int strtab, size_t off)
 static void
 elf_print_ehdr(struct elfdump *ed)
 {
+
+	if (!STAILQ_EMPTY(&ed->snl))
+		return;
+
 	if (ed->flags & SOLARIS_FMT) {
 		PRT("\nELF Header\n");
 		PRT("  ei_magic:   { %#x, %c, %c, %c }\n",
@@ -1499,19 +1529,21 @@ elf_print_phdr(struct elfdump *ed)
 {
 	GElf_Phdr	 ph;
 	size_t		 phnum;
-	int		 i;
+	int		 header, i;
 
 	if (elf_getphnum(ed->elf, &phnum) == 0) {
 		warnx("elf_getphnum failed: %s", elf_errmsg(-1));
 		return;
 	}
-	if ((ed->flags & SOLARIS_FMT) == 0)
-		PRT("\nprogram header:\n");
+	header = 0;
 	for (i = 0; (u_int64_t) i < phnum; i++) {
 		if (gelf_getphdr(ed->elf, i, &ph) != &ph) {
 			warnx("elf_getphdr failed: %s", elf_errmsg(-1));
 			continue;
 		}
+		if (!STAILQ_EMPTY(&ed->snl) &&
+		    find_name(ed, p_types[ph.p_type & 0x7]) == NULL)
+			continue;
 		if (ed->flags & SOLARIS_FMT) {
 			PRT("\nProgram Header[%d]:\n", i);
 			PRT("    p_vaddr:      %#-14jx", (uintmax_t)ph.p_vaddr);
@@ -1525,6 +1557,10 @@ elf_print_phdr(struct elfdump *ed)
 			    (uintmax_t)ph.p_offset);
 			PRT("  p_align:    %#jx\n", (uintmax_t)ph.p_align);
 		} else {
+			if (!header) {
+				PRT("\nprogram header:\n");
+				header = 1;
+			}
 			PRT("\n");
 			PRT("entry: %d\n", i);
 			PRT("\tp_type: %s\n", p_types[ph.p_type & 0x7]);
@@ -1548,6 +1584,9 @@ elf_print_shdr(struct elfdump *ed)
 	struct section *s;
 	int i;
 
+	if (!STAILQ_EMPTY(&ed->snl))
+		return;
+
 	if ((ed->flags & SOLARIS_FMT) == 0)
 		PRT("\nsection header:\n");
 	for (i = 0; (size_t)i < ed->shnum; i++) {
@@ -1955,6 +1994,9 @@ elf_print_interp(struct elfdump *ed)
 	size_t phnum;
 	int i;
 
+	if (!STAILQ_EMPTY(&ed->snl) && find_name(ed, "PT_INTERP") == NULL)
+		return;
+
 	if ((s = elf_rawfile(ed->elf, NULL)) == NULL) {
 		warnx("elf_rawfile failed: %s", elf_errmsg(-1));
 		return;
@@ -2040,6 +2082,9 @@ elf_print_got(struct elfdump *ed)
 	Elf_Data		*data, dst;
 	int			 elferr, i, len;
 
+	if (!STAILQ_EMPTY(&ed->snl))
+		return;
+
 	s = NULL;
 	for (i = 0; (size_t)i < ed->shnum; i++) {
 		s = &ed->sl[i];
@@ -2733,10 +2778,13 @@ static void
 elf_print_checksum(struct elfdump *ed)
 {
 
+	if (!STAILQ_EMPTY(&ed->snl))
+		return;
+
 	PRT("\nelf checksum: %#lx\n", gelf_checksum(ed->elf));
 }
 
-static const char *usagemsg = "\
+#define	USAGE_MESSAGE	"\
 Usage: %s [options] file...\n\
   Display information about ELF objects and ar(1) archives.\n\n\
   Options:\n\
@@ -2745,6 +2793,7 @@ Usage: %s [options] file...\n\
   -d                        Show dynamic symbols.\n\
   -e                        Show the ELF header.\n\
   -G                        Show the GOT.\n\
+  -H | --help               Show a usage message and exit.\n\
   -h                        Show hash values.\n\
   -i                        Show the dynamic interpreter.\n\
   -k                        Show the ELF checksum.\n\
@@ -2755,13 +2804,12 @@ Usage: %s [options] file...\n\
   -s                        Show the symbol table.\n\
   -S                        Use the Solaris elfdump format.\n\
   -v                        Show symbol-versioning information.\n\
-  -V                        Print a version identifier and exit.\n\
-  -w FILE                   Write output to \"FILE\".\n\
-";
+  -V | --version            Print a version identifier and exit.\n\
+  -w FILE                   Write output to \"FILE\".\n"
 
 static void
 usage(void)
 {
-	fprintf(stderr, usagemsg, ELFTC_GETPROGNAME());
+	fprintf(stderr, USAGE_MESSAGE, ELFTC_GETPROGNAME());
 	exit(EXIT_FAILURE);
 }
