@@ -26,7 +26,7 @@
 
 #include "_libdwarf.h"
 
-ELFTC_VCSID("$Id: libdwarf_frame.c 2070 2011-10-27 03:05:32Z jkoshy $");
+ELFTC_VCSID("$Id: libdwarf_frame.c 2529 2012-07-29 23:31:12Z kaiwang27 $");
 
 static int
 _dwarf_frame_find_cie(Dwarf_FrameSec fs, Dwarf_Unsigned offset,
@@ -179,8 +179,10 @@ _dwarf_frame_add_cie(Dwarf_Debug dbg, Dwarf_FrameSec fs, Dwarf_Section *ds,
 	char *p;
 
 	/* Check if we already added this CIE. */
-	if (_dwarf_frame_find_cie(fs, *off, NULL) != DW_DLE_NO_ENTRY)
+	if (_dwarf_frame_find_cie(fs, *off, &cie) != DW_DLE_NO_ENTRY) {
+		*off += cie->cie_length + 4;
 		return (DW_DLE_NONE);
+	}
 
 	if ((cie = calloc(1, sizeof(struct _Dwarf_Cie))) == NULL) {
 		DWARF_SET_ERROR(dbg, error, DW_DLE_MEMORY);
@@ -208,7 +210,8 @@ _dwarf_frame_add_cie(Dwarf_Debug dbg, Dwarf_FrameSec fs, Dwarf_Section *ds,
 	cie->cie_length = length;
 
 	cie->cie_version = dbg->read(ds->ds_data, off, 1);
-	if (cie->cie_version != 1 && cie->cie_version != 3) {
+	if (cie->cie_version != 1 && cie->cie_version != 3 &&
+	    cie->cie_version != 4) {
 		DWARF_SET_ERROR(dbg, error, DW_DLE_FRAME_VERSION_BAD);
 		return (DW_DLE_FRAME_VERSION_BAD);
 	}
