@@ -24,7 +24,7 @@
  * SUCH DAMAGE.
  */
 
-#include <sys/mman.h>
+#include <sys/param.h>
 #include <sys/stat.h>
 
 #include <assert.h>
@@ -37,7 +37,11 @@
 
 #include "_libelf.h"
 
-LIBELF_VCSID("$Id: elf_update.c 1922 2011-09-23 08:04:33Z jkoshy $");
+#if	ELFTC_HAVE_MMAP
+#include <sys/mman.h>
+#endif
+
+ELFTC_VCSID("$Id: elf_update.c 2348 2011-12-19 02:22:45Z jkoshy $");
 
 /*
  * Layout strategy:
@@ -107,13 +111,14 @@ static int
 _libelf_compute_section_extents(Elf *e, Elf_Scn *s, off_t rc)
 {
 	int ec;
-	size_t fsz, msz;
 	Elf_Data *d;
-	Elf32_Shdr *shdr32;
-	Elf64_Shdr *shdr64;
+	size_t fsz, msz;
 	uint32_t sh_type;
 	uint64_t d_align;
+	Elf32_Shdr *shdr32;
+	Elf64_Shdr *shdr64;
 	unsigned int elftype;
+	struct _Libelf_Data *ld;
 	uint64_t scn_size, scn_alignment;
 	uint64_t sh_align, sh_entsize, sh_offset, sh_size;
 
@@ -187,7 +192,9 @@ _libelf_compute_section_extents(Elf *e, Elf_Scn *s, off_t rc)
 	 */
 	scn_size = 0L;
 	scn_alignment = 0;
-	STAILQ_FOREACH(d, &s->s_data, d_next)  {
+	STAILQ_FOREACH(ld, &s->s_data, d_next)  {
+
+		d = &ld->d_data;
 
 		/*
 		 * The data buffer's type is known.
@@ -699,12 +706,13 @@ static size_t
 _libelf_write_scn(Elf *e, char *nf, struct _Elf_Extent *ex)
 {
 	int ec;
-	size_t fsz, msz, nobjects, rc;
-	uint32_t sh_type;
-	uint64_t sh_off, sh_size;
-	int elftype;
 	Elf_Scn *s;
+	int elftype;
 	Elf_Data *d, dst;
+	uint32_t sh_type;
+	struct _Libelf_Data *ld;
+	uint64_t sh_off, sh_size;
+	size_t fsz, msz, nobjects, rc;
 
 	assert(ex->ex_type == ELF_EXTENT_SECTION);
 
@@ -744,7 +752,10 @@ _libelf_write_scn(Elf *e, char *nf, struct _Elf_Extent *ex)
 		if ((d = elf_rawdata(s, NULL)) == NULL)
 			return ((off_t) -1);
 
-		STAILQ_FOREACH(d, &s->s_rawdata, d_next) {
+		STAILQ_FOREACH(ld, &s->s_rawdata, d_next) {
+
+			d = &ld->d_data;
+
 			if ((uint64_t) rc < sh_off + d->d_off)
 				(void) memset(nf + rc,
 				    LIBELF_PRIVATE(fillchar), sh_off +
@@ -772,7 +783,9 @@ _libelf_write_scn(Elf *e, char *nf, struct _Elf_Extent *ex)
 
 	dst.d_version = e->e_version;
 
-	STAILQ_FOREACH(d, &s->s_data, d_next) {
+	STAILQ_FOREACH(ld, &s->s_data, d_next) {
+
+		d = &ld->d_data;
 
 		msz = _libelf_msize(d->d_type, ec, e->e_version);
 
@@ -1047,6 +1060,7 @@ _libelf_write_elf(Elf *e, off_t newsize, struct _Elf_Extent_List *extents)
 			LIBELF_SET_ERROR(IO, errno);
 			goto error;
 		}
+#if	ELFTC_HAVE_MMAP
 		if (e->e_flags & LIBELF_F_RAWFILE_MMAP) {
 			assert(e->e_rawfile != NULL);
 			assert(e->e_cmd == ELF_C_RDWR);
@@ -1055,6 +1069,7 @@ _libelf_write_elf(Elf *e, off_t newsize, struct _Elf_Extent_List *extents)
 				goto error;
 			}
 		}
+#endif
 	}
 
 	/*
@@ -1071,18 +1086,23 @@ _libelf_write_elf(Elf *e, off_t newsize, struct _Elf_Extent_List *extents)
 	 */
 	if (e->e_cmd == ELF_C_RDWR) {
 		assert(e->e_rawfile != NULL);
-		if (e->e_flags & LIBELF_F_RAWFILE_MMAP) {
+		assert((e->e_flags & LIBELF_F_RAWFILE_MALLOC) ||
+		    (e->e_flags & LIBELF_F_RAWFILE_MMAP));
+		if (e->e_flags & LIBELF_F_RAWFILE_MALLOC) {
+			free(e->e_rawfile);
+			e->e_rawfile = newfile;
+			newfile = NULL;
+		}
+#if	ELFTC_HAVE_MMAP
+		else if (e->e_flags & LIBELF_F_RAWFILE_MMAP) {
 			if ((e->e_rawfile = mmap(NULL, (size_t) newsize,
 			    PROT_READ, MAP_PRIVATE, e->e_fd, (off_t) 0)) ==
 			    MAP_FAILED) {
 				LIBELF_SET_ERROR(IO, errno);
 				goto error;
 			}
-		} else if (e->e_flags & LIBELF_F_RAWFILE_MALLOC) {
-			free(e->e_rawfile);
-			e->e_rawfile = newfile;
-			newfile = NULL;
 		}
+#endif	/* ELFTC_HAVE_MMAP */
 
 		/* Record the new size of the file. */
 		e->e_rawsize = newsize;
@@ -1176,7 +1196,7 @@ elf_update(Elf *e, Elf_Cmd c)
 		goto done;
 	}
 
-	return (_libelf_write_elf(e, rc, &extents));
+	rc = _libelf_write_elf(e, rc, &extents);
 
 done:
 	_libelf_release_extents(&extents);
