@@ -48,7 +48,7 @@
 
 #include "_elftc.h"
 
-ELFTC_VCSID("$Id: nm.c 2164 2011-11-13 05:23:18Z jkoshy $");
+ELFTC_VCSID("$Id: nm.c 2484 2012-04-07 15:52:05Z kaiwang27 $");
 
 /* symbol information list */
 STAILQ_HEAD(sym_head, sym_entry);
@@ -193,7 +193,6 @@ static bool		is_sec_nobits(GElf_Shdr *);
 static bool		is_sec_readonly(GElf_Shdr *);
 static bool		is_sec_text(GElf_Shdr *);
 static void		print_ar_index(int, Elf *);
-static void		print_demangle_name(const char *, const char *);
 static void		print_header(const char *, const char *);
 static void		print_version(void);
 static int		read_elf(Elf *, const char *, Elf_Kind);
@@ -426,7 +425,7 @@ parse_demangle_option(const char *opt)
 {
 
 	if (opt == NULL)
-		return (0);
+		return (ELFTC_DEM_UNKNOWN);
 	else if (!strncasecmp(opt, "gnu-v2", 6))
 		return (ELFTC_DEM_GNU2);
 	else if (!strncasecmp(opt, "gnu-v3", 6))
@@ -444,12 +443,14 @@ static void
 get_opt(int argc, char **argv)
 {
 	int ch;
+	bool is_posix, oflag;
 
 	if (argc <= 0 || argv == NULL)
 		return;
 
+	oflag = is_posix = false;
 	nm_opts.t = RADIX_HEX;
-	while ((ch = getopt_long(argc, argv, "ABCDSVPaefghlnoprst:uvx",
+	while ((ch = getopt_long(argc, argv, "ABCDF:PSVaefghlnoprst:uvx",
 		    nm_longopts, NULL)) != -1) {
 		switch (ch) {
 		case 'A':
@@ -461,6 +462,9 @@ get_opt(int argc, char **argv)
 		case 'C':
 			nm_opts.demangle_type = parse_demangle_option(optarg);
 			break;
+		case 'D':
+			nm_opts.print_symbol = PRINT_SYM_DYN;
+			break;
 		case 'F':
 			/* sysv, bsd, posix */
 			switch (optarg[0]) {
@@ -470,6 +474,7 @@ get_opt(int argc, char **argv)
 				break;
 			case 'P':
 			case 'p':
+				is_posix = true;
 				nm_opts.elem_print_fn =
 				    &sym_elem_print_all_portable;
 				break;
@@ -484,8 +489,9 @@ get_opt(int argc, char **argv)
 			}
 
 			break;
-		case 'D':
-			nm_opts.print_symbol = PRINT_SYM_DYN;
+		case 'P':
+			is_posix = true;
+			nm_opts.elem_print_fn = &sym_elem_print_all_portable;
 			break;
 		case 'S':
 			nm_opts.print_size = 1;
@@ -493,22 +499,29 @@ get_opt(int argc, char **argv)
 		case 'V':
 			print_version();
 			/* NOTREACHED */
-		case 'P':
-			nm_opts.elem_print_fn = &sym_elem_print_all_portable;
-			break;
 		case 'a':
 			nm_opts.print_debug = true;
 			break;
-		case 'f':
-			break;
 		case 'e':
 			filter_insert(sym_elem_global_static);
 			break;
+		case 'f':
+			break;
 		case 'g':
 			filter_insert(sym_elem_global);
 			break;
+		case 'h':
+			usage(0);
+			break;
+		case 'l':
+			nm_opts.debug_line = true;
+			break;
+		case 'n':
+		case 'v':
+			nm_opts.sort_fn = &cmp_value;
+			break;
 		case 'o':
-			nm_opts.t = RADIX_OCT;
+			oflag = true;
 			break;
 		case 'p':
 			nm_opts.sort_fn = &cmp_none;
@@ -540,13 +553,7 @@ get_opt(int argc, char **argv)
 			filter_insert(sym_elem_undef);
 			nm_opts.undef_only = true;
 			break;
-		case 'l':
-			nm_opts.debug_line = true;
-			break;
-		case 'n':
-		case 'v':
-			nm_opts.sort_fn = &cmp_value;
-			break;
+		/* case 'v': see case 'n' above. */
 		case 'x':
 			nm_opts.t = RADIX_HEX;
 			break;
@@ -561,13 +568,23 @@ get_opt(int argc, char **argv)
 			if (nm_opts.no_demangle != 0)
 				nm_opts.demangle_type = -1;
 			break;
-		case 'h':
-			usage(0);
 		default :
 			usage(1);
 		}
 	}
 
+	/*
+	 * In POSIX mode, the '-o' option controls the output radix.
+	 * In non-POSIX mode, the option is a synonym for the '-A' and
+	 * '--print-file-name' options.
+	 */
+	if (oflag) {
+		if (is_posix)
+			nm_opts.t = RADIX_OCT;
+		else
+			nm_opts.print_name = PRINT_NAME_FULL;
+	}
+
 	assert(nm_opts.sort_fn != NULL && "nm_opts.sort_fn is null");
 	assert(nm_opts.elem_print_fn != NULL &&
 	    "nm_opts.elem_print_fn is null");
@@ -845,20 +862,16 @@ print_ar_index(int fd, Elf *arf)
 	elf_rand(arf, start);
 }
 
-static void
-print_demangle_name(const char *format, const char *name)
-{
-	char dem[8192];
-
-	if (format == NULL || name == NULL)
-		return;
-
-	if (nm_opts.demangle_type < 0 ||
-	    elftc_demangle(name, dem, sizeof(dem), nm_opts.demangle_type) < 0)
-		printf(format, name);
-	else
-		printf(format, dem);
-}
+#define	DEMANGLED_BUFFER_SIZE	(8 * 1024)
+#define	PRINT_DEMANGLED_NAME(FORMAT, NAME) do {				\
+	char _demangled[DEMANGLED_BUFFER_SIZE];				\
+	if (nm_opts.demangle_type < 0 ||				\
+	    elftc_demangle((NAME), _demangled, sizeof(_demangled),	\
+		nm_opts.demangle_type) < 0)				\
+		printf((FORMAT), (NAME));				\
+	else								\
+		printf((FORMAT), _demangled);				\
+	} while (0)
 
 static void
 print_header(const char *file, const char *obj)
@@ -1355,7 +1368,7 @@ read_elf(Elf *elf, const char *filename, Elf_Kind kind)
 process_sym:
 
 	p_data.list_num = get_sym(elf, &list_head, shnum, dynndx, strndx,
-	    type_table, (const char **)sec_table, shnum);
+	    type_table, (void *) sec_table, shnum);
 
 	if (p_data.list_num == 0)
 		goto next_cmd;
@@ -1363,7 +1376,7 @@ process_sym:
 	p_data.headp = &list_head;
 	p_data.sh_num = shnum;
 	p_data.t_table = type_table;
-	p_data.s_table = (const char **)sec_table;
+	p_data.s_table = (void *) sec_table;
 	p_data.filename = filename;
 	p_data.objname = objname;
 
@@ -1379,6 +1392,8 @@ next_cmd:
 				free(func->name);
 				free(func);
 			}
+			free(func_info);
+			func_info = NULL;
 		}
 		if (var_info != NULL) {
 			while (!SLIST_EMPTY(var_info)) {
@@ -1388,6 +1403,8 @@ next_cmd:
 				free(var->name);
 				free(var);
 			}
+			free(var_info);
+			var_info = NULL;
 		}
 		if (line_info != NULL) {
 			while (!SLIST_EMPTY(line_info)) {
@@ -1608,7 +1625,7 @@ sym_elem_print_all(char type, const char *sec, const GElf_Sym *sym,
 	}
 
 	printf(" %c ", type);
-	print_demangle_name("%s", name);
+	PRINT_DEMANGLED_NAME("%s", name);
 }
 
 static void
@@ -1620,7 +1637,7 @@ sym_elem_print_all_portable(char type, const char *sec, const GElf_Sym *sym,
 	    nm_opts.value_print_fn == NULL)
 		return;
 
-	print_demangle_name("%s", name);
+	PRINT_DEMANGLED_NAME("%s", name);
 	printf(" %c ", type);
 	if (!IS_UNDEF_SYM_TYPE(type)) {
 		nm_opts.value_print_fn(sym);
@@ -1640,7 +1657,7 @@ sym_elem_print_all_sysv(char type, const char *sec, const GElf_Sym *sym,
 	    nm_opts.value_print_fn == NULL)
 		return;
 
-	print_demangle_name("%-20s|", name);
+	PRINT_DEMANGLED_NAME("%-20s|", name);
 	if (IS_UNDEF_SYM_TYPE(type))
 		printf("                ");
 	else
