@@ -39,7 +39,7 @@
 
 #include "_elftc.h"
 
-ELFTC_VCSID("$Id: size.c 2124 2011-11-09 16:13:53Z jkoshy $");
+ELFTC_VCSID("$Id: size.c 2350 2011-12-19 10:20:06Z jkoshy $");
 
 #define	BUF_SIZE			1024
 #define	ELF_ALIGN(val,x) (((val)+(x)-1) & ~((x)-1))
@@ -88,7 +88,7 @@ static struct option size_longopts[] = {
 	{ "help",	no_argument,	NULL,	'h' },
 	{ "radix",	required_argument, &size_option, OPT_RADIX },
 	{ "totals",	no_argument,	NULL,	't' },
-	{ "version",	no_argument,	NULL,	'v' },
+	{ "version",	no_argument,	NULL,	'V' },
 	{ NULL, 0, NULL, 0 }  
 };
 
@@ -194,7 +194,7 @@ main(int argc, char **argv)
 	argc -= optind;
 	argv += optind;
 
-	files = (argc == 0) ? default_args : (const char **) argv;
+	files = (argc == 0) ? default_args : (void *) argv;
 
 	while ((fn = *files) != NULL) {
 		rc = handle_elf(fn);
@@ -873,13 +873,14 @@ tbl_flush(void)
 		}
 		free(tb->tbl[i]);
 	}
+	free(tb->tbl);
 	free(tb->width);
 	free(tb);
 	tb = NULL;
 }
 
-static const char *usagemsg = "\
-Usage: size [options] file ...\n\
+#define	USAGE_MESSAGE	"\
+Usage: %s [options] file ...\n\
   Display sizes of ELF sections.\n\n\
   Options:\n\
   --format=format    Display output in specified format.  Supported\n\
@@ -896,12 +897,12 @@ Usage: size [options] file ...\n\
   -h                 Same as option --help.\n\
   -o                 Equivalent to `--radix=8'.\n\
   -t                 Equivalent to option --totals.\n\
-  -x                 Equivalent to `--radix=16'.\n";
+  -x                 Equivalent to `--radix=16'.\n"
 
 static void
 usage(void)
 {
-	(void) fprintf(stderr, "%s", usagemsg);
+	(void) fprintf(stderr, USAGE_MESSAGE, ELFTC_GETPROGNAME());
 	exit(EXIT_FAILURE);
 }
 
